<?php
/**
 * @author Abhimanyu Sharma <abhimanyusharma003@gmail.com>
 */
namespace Hackersnews\Repository\Eloquent;

use Carbon\Carbon;
use DB;
use Str;
use Post;
use Auth;
use Category;
use Favorite;
use Purifier;
use Hackersnews\Repository\PostsRepositoryInterface;

class PostsRepository extends AbstractRepository implements PostsRepositoryInterface {

    /**
     * @var \Post
     */
    protected $model;

    /**
     * @param Post     $posts
     * @param Category $category
     */
    public function __construct(Post $posts, Category $category)
    {
        $this->model = $posts;
        $this->category = $category;
    }

    /**
     * @param      $id
     * @param null $type
     * @return mixed
     */
    public function getById($id, $type = null)
    {
        $post = $this->posts($type)->whereId($id)->first();

        return $post;
    }

    /**
     * @param null $type
     * @param null $category
     * @return Post
     */
    private function posts($type = null, $category = null)
    {
        if ($category && $type)
        {
            $cat = $this->category->whereSlug($category)->first();
            if ($cat) return $this->model->approved()->whereCategoryId($cat->id)->whereType($type);
            return $this->model->approved()->whereCategoryId(null)->whereType($type);
        }
        elseif ($category)
        {
            $cat = $this->category->whereSlug($category)->first();
            if ($cat) return $this->model->approved()->whereCategoryId($cat->id);

            return $this->model->approved()->whereCategoryId(null)->whereType($type);
        }
        elseif ($type)
        {
            return $this->model->approved()->whereType($type);
        }

        return $this->model->approved();
    }

    /**
     * @param $id
     * @return mixed
     */
    public function getAnyById($id)
    {
        $post = $this->posts()->whereId($id)->first();

        return $post;
    }


    /**
     * @param null $type
     * @param null $category
     * @return mixed
     */
    public function getTrending($type = null, $category = null)
    {
        $posts = $this->posts($type, $category)->with('comments', 'votes', 'category', 'user', 'votes.user')
            ->leftJoin('votes', 'posts.id', '=', 'votes.post_id')
            ->leftJoin('comments', 'posts.id', '=', 'comments.post_id')
            ->select('posts.*', DB::raw('count(comments.post_id)*7 + count(votes.post_id)*5 + posts.views as popular'))
            ->groupBy('posts.id')->with('user')->orderBy('popular', 'desc')->whereMonth('posts.created_at', '>=', date('n', strtotime('-1 month')))
            ->paginate(perPage());

        return $posts;
    }

    /**
     * @param null   $type
     * @param null   $category
     * @param string $order
     * @return mixed
     */
    public function getLatest($type = null, $category = null, $order = 'desc')
    {
        $posts = $this->posts($type, $category)->with('comments', 'votes', 'category', 'user', 'votes.user')->orderBy('created_at', $order)->paginate(perPage());

        return $posts;
    }

    /**
     * @param null $type
     * @param null $category
     * @return mixed
     */
    public function getFeatured($type = null, $category = null)
    {
        $posts = $this->posts($type, $category)->whereNotNull('featured_at')->with('comments', 'votes', 'category', 'user', 'votes.user')->orderBy('featured_at', 'desc')->paginate(perPage());

        return $posts;
    }


    /**
     * @param $news
     * @return mixed
     */
    public function incrementViews($news)
    {
        $news->views = $news->views + 1;
        $news->save();

        return $news;
    }

    /**
     * @param      $term
     * @param null $type
     * @param null $category
     * @return mixed
     */
    public function search($term, $type = null, $category = null)
    {
        $posts = $this->posts($type, $category)->where(function ($query) use ($term)
        {
            $query->where('title', 'LIKE', '%' . $term . '%')
                ->orWhere('summary', 'LIKE', '%' . $term . '%')
                ->orWhere('url', 'LIKE', '%' . $term . '%');
        })->with('comments', 'votes', 'category', 'user', 'votes.user')->orderBy('title', 'asc')->paginate(perPage());

        return $posts;
    }

    /**
     * @param       $type
     * @param array $input
     * @return mixed
     */
    public function create($type, array $input)
    {
        $post = $this->getNew();
        $post->title = $input['title'];
        $post->summary = Purifier::clean($input['summary']);
        $post->user_id = Auth::user()->id;
        $post->category_id = $input['category'];
        $slug = @Str::slug($input['title']);
        if ( ! $slug)
        {
            $slug = Str::random(9);
        }
        if ((bool) autoApprove() == false)
        {
            $post->approved_at = null;
        }
        else
        {
            $post->approved_at = Carbon::now();
        }
        if ($type == 'question')
        {
            $post->type = 'question';
            $post->url = null;
        }
        elseif ($type == 'news')
        {
            $post->type = 'news';
            $post->url = filter_var($input['url'], FILTER_SANITIZE_URL);
        }
        $post->slug = $slug;
        $post->save();

        return $post;
    }


    /**
     * @param Post  $posts
     * @param array $input
     * @return Post
     */
    public function edit(Post $posts, array $input)
    {
        $post = $posts;
        if (isset($input['delete']))
        {
            $post->flags()->delete();
            $post->comments()->delete();
            $post->reply()->delete();
            $post->votes()->delete();
            $post->delete();

            return $posts;
        }
        $slug = @Str::slug($input['title']);
        if ( ! $slug)
        {
            $slug = Str::random(9);
        }
        $post->title = $input['title'];
        $post->slug = $slug;
        $post->summary = Purifier::clean($input['summary']);
        $post->category_id = $input['category'];
        if ($post->type == 'news')
        {
            $post->url = $input['url'];
        }
        if ((bool) autoApprove() == false)
        {
            $post->approved_at = null;
        }
        $post->update();

        return $post;
    }

    //@todo add filter for admin panel
    /**
     * @return mixed
     */
    public function getApprovalRequired()
    {
        $posts = $this->model->whereNull('approved_at')->with('comments', 'votes', 'category', 'user', 'votes.user')->orderBy('featured_at', 'desc')->paginate(perPage());

        return $posts;
    }

    /**
     * Admin helper, gives output irr-respective of post approved or not
     *
     * @param $id
     * @return mixed
     */
    public function getByIdForAdmin($id)
    {
        $posts = $this->model->whereId($id)->first();

        return $posts;
    }

    /**
     * Admin helper, to edit post irr-respective of post approved or not
     *
     * @param Post  $posts
     * @param array $input
     * @return Post
     */
    public function adminEdit(Post $posts, array $input)
    {
        $post = $posts;
        if (isset($input['delete']))
        {
            $post->flags()->delete();
            $post->comments()->delete();
            $post->reply()->delete();
            $post->votes()->delete();
            $post->delete();

            return $posts;
        }
        $slug = @Str::slug($input['title']);
        if ( ! $slug)
        {
            $slug = Str::random(9);
        }
        $post->title = $input['title'];
        $post->slug = $slug;
        $post->summary = Purifier::clean($input['summary']);
        $post->category_id = $input['category'];
        if ($post->type == 'news')
        {
            $post->url = $input['url'];
        }
        if (isset($input['approve']))
        {
            $post->approved_at = Carbon::now();
        }
        if (isset($input['featured']) && $post->featured_at == null)
        {
            $post->featured_at = Carbon::now();
        }
        elseif ( ! isset($input['featured']))
        {
            $post->featured_at = null;
        }
        $post->update();

        return $post;
    }
}